# 输出图形到SVG临时文件以支持中文标注
# https://stackoverflow.com/questions/39093777/renderimage-and-svg-in-shiny-app/39263320#39263320
# 多图层图例设置参阅
# https://stackoverflow.com/questions/18394391/custom-legend-for-multiple-layer-ggplot
# Shiny data table国际化语言支持参阅
# https://rstudio.github.io/DT/004-i18n.html
# Shiny Data Table的操作可以参考
# https://rstudio.github.io/DT/shiny.html
# 选中行的操作可以参考
# https://yihui.shinyapps.io/DT-rows/
shinyServer(function(input, output, session) {
  
  algo<- reactive({
    input$algo
  })
  
  threshold<- reactive({
    input$threshold
  })
  
  # 更新由宿主页面传入的参数
  observe({
    print(algo())
    # print(threshold())
    # 更新由宿主页面传入的参数
    updateTextInput(session, "algo", value = algo())
    # 注意这里用updateTextInput更新文本到浏览器端，如果用updateSelectInput()会报错，可能要求的是下标。    
    # updateSelectInput(session, "algo", value = algo())
  })
  
  output$parameters<-renderText({
    paste("算法：",algo(),"        异常值阀值：",threshold(),"%",sep="")
  })

  output$seconds<-renderText({
    seconds
  })
  
  output$performance<- renderTable({
    names(perf)<-c("算法","训练集","验证集","耗时")
    perf
  },
  digits = 3
  )
  
  output$trainPlot<-renderImage({
    colName<- algos[algo()]
    data<- data.frame(origin = train["origin"], predict = train[colName])
    names(data)<-c("origin","predict")
    data<-data[order(data["origin"]),]
    data["index"]<-1:nrow(data)
    data["upper"]<-data["origin"]+log(1+threshold()/100)
    data["lower"]<-data["origin"]+log(1-threshold()/100)

    image<-ggplot(data = data)+
              geom_point(mapping = aes(x = index, y = predict, color = "blue"), size=0.1)+
              geom_line(mapping = aes(x = index, y = origin, color = "red"))+
              geom_ribbon(aes(index, ymax=upper, ymin=lower, fill="yellow"), alpha=0.25)+
              ggtitle("训练集拟合效果图")+
              xlab("样本")+
              ylab("ln(房价)")+
              scale_fill_identity(name = '异常值区间', guide = 'legend',labels = c('正常值')) +
              scale_colour_manual(name = '颜色', 
                          values =c('blue'='blue','red'='red'), labels = c('预测值','实际值'))
    
    # 输出SVG到临时文件
    # This file will be removed later by renderImage
    outfile <- tempfile(fileext='.svg')
    #This actually save the plot in a image
    # ggsave(file=outfile, plot=image, width=mysvgwidth, height=mysvgheight)
    # 单位是英寸
    ggsave(file=outfile, plot=image, width=6, height=4)
    
    
    # Return a list containing the filename
    list(src = normalizePath(outfile),
         contentType = 'image/svg+xml',
         # 单位是像素
         #width = 400,
         height = 400,
         alt = "训练集拟合效果图")    
  },
  # SVG图发到客户端后，renderImage删除临时文件
  deleteFile =  TRUE
  )
  
  output$validPlot<-renderImage({
    colName<- algos[algo()]
    data<- data.frame(origin = valid["origin"], predict = valid[colName])
    names(data)<-c("origin","predict")
    data<-data[order(data["origin"]),]
    data["index"]<-1:nrow(data)
    data["upper"]<-data["origin"]+log(1+threshold()/100)
    data["lower"]<-data["origin"]+log(1-threshold()/100)
    
    image<-ggplot(data = data)+
      geom_point(mapping = aes(x = index, y = predict, color = "blue"), size=0.1)+
      geom_line(mapping = aes(x = index, y = origin, color = "red"))+
      geom_ribbon(aes(index, ymax=upper, ymin=lower, fill="yellow"), alpha=0.25)+
      ggtitle("验证集拟合效果图")+
      xlab("样本")+
      ylab("ln(房价)")+
      scale_fill_identity(name = '异常值区间', guide = 'legend',labels = c('正常值')) +
      scale_colour_manual(name = '颜色', 
                          values =c('blue'='blue','red'='red'), labels = c('预测值','实际值'))
    

    # 输出SVG到临时文件
    # This file will be removed later by renderImage
    outfile <- tempfile(fileext='.svg')
    #This actually save the plot in a image
    #ggsave(file=outfile, plot=image, width=mysvgwidth, height=mysvgheight)
    ggsave(file=outfile, plot=image, width=6, height=4)
    
    
    # Return a list containing the filename
    list(src = normalizePath(outfile),
         contentType = 'image/svg+xml',
         #width = width,
         height = 400,
         alt = "验证集拟合效果图")    
  },
  # SVG图发到客户端后，renderImage删除临时文件
  deleteFile =  TRUE
  )  
  
  output$outliers<- renderDataTable({
    colName<- algos[algo()]
    data<- data.frame(origin = valid["origin"], predict = valid[colName])
    names(data)<-c("origin","predict")
    data["outlier"]<- FALSE
    data[which(data["predict"] < data["origin"]+log(1-threshold()/100)),"outlier"] <-TRUE
    data[which(data["predict"] > data["origin"]+log(1+threshold()/100)),"outlier"] <-TRUE
    valid_X3<- valid_X2[,c("Lattitude","Longtitude","Distance","BuildingArea","Landsize","YearBuilt","Year")]
    valid_X3["Type"]<-valid_X2["Type_h"]*4+valid_X2["Type_t"]*2+valid_X2["Type_u"]
    valid_X3[which(valid_X3["Type"]==4),"Type"]<-"H"
    valid_X3[which(valid_X3["Type"]==2),"Type"]<-"T"
    valid_X3[which(valid_X3["Type"]==1),"Type"]<-"U"
    #valid_X3["Origin"]<-data["origin"]
    #valid_X3["Predict"]<-data["predict"]
    # 转换为原量纲
    valid_X3["Origin"]<-round(exp(data["origin"]),2)
    valid_X3["Predict"]<-round(exp(data["predict"]),2)
    outliers<-valid_X3[which(data["outlier"]==TRUE),]
    outliers["SE"]<-round((outliers["Predict"] - outliers["Origin"])/outliers["Origin"],3)
    #outliers["ID"]<- row.names(outliers)
    outliers<-outliers[order(outliers["SE"]),]
    # 放入 session 中以便后面选择行时引用，不能用行索引，它不是全局的， 行名才是全局的。
    session$userData$outliers<- outliers
  },
  #rownames = FALSE, 
  options = list(
    pageLength = 10,
    #language = list(url = '//cdn.datatables.net/plug-ins/1.10.11/i18n/Chinese.json')  
    language = list(url = 'Chinese.json')   
    )
  )

  # 返回并显示选中行的行名，行名是全局的，用于定位对应的行，更新浏览器端显示。
  observe({
    outliers<- session$userData$outliers
    rows<- outliers[input$outliers_rows_selected,]
    outliersStr<- paste(row.names(rows), collapse=",")
    updateTextAreaInput(session, "rows_selected", value = outliersStr) 
  })
  
})